$set 15 #Common

$ #Yes
# 네
$ #No
# 아니오

$ #DirectionTitle
# 방향
$ #DirectionHoriz
# 가로
$ #DirectionVert
# 세로

$ #AlwaysOnTop
# 항상 위

$ #PlacementTitle
# 위치
$ #PlacementTopLeft
# 왼쪽 위
$ #PlacementCenterLeft
# 왼쪽 가운데
$ #PlacementBottomLeft
# 왼쪽 아래
$ #PlacementTopCenter
# 가운데 위
$ #PlacementBottomCenter
# 가운데 아래
$ #PlacementTopRight
# 오른쪽 위
$ #PlacementCenterRight
# 오른쪽 가운데
$ #PlacementBottomRight
# 오른쪽 아래

$ #AutoHide
# 자동으로 숨기기
