$set 1 #BaseDisplay

$ #XError
# %s:  помилка X сервера: %s(%d) значення %d/%d\n ресурс 0x%lx\n
$ #SignalCaught
# %s: отриманий сигнал %d \n
$ #ShuttingDown
# вимкнення\n
$ #Aborting
# переривання... створюється дамп core\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: не вдалося з'єднання з X сервером.\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: нема можливості позначити активний дисплей як "закритий-на-виконання"\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): погане вікно видаляється з черги подій\n
