$set 9 #Window


$ #Creating
# BlackboxWindow::BlackboxWindow: Génération 0x%lx\n
$ #XGetWindowAttributesFail
# BlackboxWindow::BlackboxWindow: Echec de XGetWindowAttributres\n
$ #CannotFindScreen
# BlackboxWindow::BlackboxWindow: Impossible de trouver l'écran de la fenêtre racine 0x%lx\n
$ #Unnamed
# Inconnu
$ #MapRequest
# BlackboxWindow::mapRequestEvent() pour 0x%lx\n
$ #UnmapNotify
# BlackboxWindow::unmapNotifyEvent() pour 0x%lx\n
$ #ReparentNotify
# BlackboxWindow::reparentNotifyEvent: réattribue 0x%lx à la 0x%lx\n
