$set 15 #Common

$ #Yes
# 正確
$ #No
# 取消 

$ #DirectionTitle
# 方向
$ #DirectionHoriz
# 橫向 
$ #DirectionVert
# 直向 

$ #AlwaysOnTop
# 永遠在最上層

$ #PlacementTitle
# 放置 
$ #PlacementTopLeft
# 左上 
$ #PlacementCenterLeft
# 左邊置中
$ #PlacementBottomLeft
# 左下 
$ #PlacementTopCenter
# 最上置中
$ #PlacementBottomCenter
# 最下置中 
$ #PlacementTopRight
# 右上 
$ #PlacementCenterRight
# 右邊置中 
$ #PlacementBottomRight
# 右下 

$ #AutoHide
# 自動隱藏 
