$set 10 #Windowmenu

$ #SendTo
# Skicka till...
$ #Shade
# Skugga
$ #Iconify
# Ikonifiera
$ #Maximize
# Maximera
$ #Raise
# Höj
$ #Lower
# Sänk
$ #Stick
# Klibbig
$ #KillClient
# Döda klient
$ #Close
# Stäng
