$set 10 #Windowmenu

$ #SendTo
# Envoie vers ...
$ #Shade
# Fantôme
$ #Iconify
# Iconifie
$ #Maximize
# Maximise
$ #Raise
# Élève
$ #Lower
# Abaisse
$ #Stick
# Épingle
$ #KillClient
# Massacre
$ #Close
# Ferme
