$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: 查询X服务器时出错.\n  \
已经有另一个窗口管理器在运行 %s.\n
$ #ManagingScreen
# BScreen::BScreen: 对屏幕 %d 使用视 0x%lx, 深度 %d\n
$ #FontLoadFail
# BScreen::LoadStyle(): 无法载入字体 '%s'\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): 无法载入默认字体.\n
$ #EmptyMenuFile
# %s: 清空菜单文件\n
$ #xterm
# xterm
$ #Restart
# 重新开始
$ #Exit
# 退出
$ #EXECError
# BScreen::parseMenuFile: [exec] 错误, 没有菜单的标题或命令\n
$ #EXITError
# BScreen::parseMenuFile: [exit] 错误, 没有菜单的标题\n
$ #STYLEError
# BScreen::parseMenuFile: [style] 错误, 没有菜单标题或文件名\n
$ #CONFIGError
# BScreen::parseMenuFile: [config] 错误, 没有菜单标题\n
$ #INCLUDEError
# BScreen::parseMenuFile: [include] 错误, 没有文件名\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: [include] 错误, '%s' 不是一个普通文件\n
$ #SUBMENUError
# BScreen::parseMenuFile: [submenu] 错误, 没有菜单标题\n
$ #RESTARTError
# BScreen::parseMenuFile: [restart] 错误, 没有菜单标题\n
$ #RECONFIGError
# BScreen::parseMenuFile: [reconfig] 错误, 没有菜单标题\n
$ #STYLESDIRError
# BScreen::parseMenuFile: [stylesdir/stylesmenu] 错误, 没有指名目录\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: [stylesdir/stylesmenu] 错误, '%s' 不是目录\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: [stylesdir/stylesmenu] 错误, '%s' 不存在\n
$ #WORKSPACESError
# BScreen::parseMenuFile: [workspaces] 错误, 没有菜单标题\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# 左: %4d x 上: %4d
$ #GeometryFormat
# 宽: %4d x 高: %4d

