$set 15 #Common

$ #Yes
# Ja
$ #No
# Nej

$ #DirectionTitle
# Retning
$ #DirectionHoriz
# Horisontal
$ #DirectionVert
# Vertikal

$ #AlwaysOnTop
# Altid øverst

$ #PlacementTitle
# Placering
$ #PlacementTopLeft
# Øverste venstre hjørne
$ #PlacementCenterLeft
# Venstre i midten
$ #PlacementBottomLeft
# Bunden til venstre
$ #PlacementTopCenter
# Toppen i midten
$ #PlacementBottomCenter
# Bunden i midten
$ #PlacementTopRight
# Øverst til højre
$ #PlacementCenterRight
# Til højre i midten
$ #PlacementBottomRight
# Til højre i bunden

$ #AutoHide
# Gem automatisk
