$set 14 #main

$ #RCRequiresArg
# помилка: "-rc" вимагає аргумент\n
$ #DISPLAYRequiresArg
# помилка: "-display" вимагає аргумент\n
$ #WarnDisplaySet
# попередження: не вдалося встановити змінну середовища "DISPLAY"\n
$ #Usage
# Blackbox %s: (c) 1997 - 2000 Brad Hughes\n\n\
  -display <string>\t\tобслуговувати вказаний дисплей.\n\
  -rc <string>\t\t\tвикористовувати альтернативний файл ресурсів.\n\
  -version\t\t\tвивести номер версії і вийти.\n\
  -help\t\t\t\tвивести цю підказку і вийти.\n\n
$ #CompileOptions
# Compile time options:\n\
  Debugging\t\t\t%s\n\
  Interlacing:\t\t\t%s\n\
  Shape:\t\t\t%s\n\
  Slit:\t\t\t\t%s\n\
  8bpp Ordered Dithering:\t%s\n\n
