$set 9 #Window


$ #Creating
# BlackboxWindow::BlackboxWindow: izveidoju 0x%lx\n
$ #XGetWindowAttributesFail
# BlackboxWindow::BlackboxWindow: XGetWindowAttributres neizdevās\n
$ #CannotFindScreen
# BlackboxWindow::BlackboxWindow: nevaru atrast ekrānu saknes logam 0x%lx\n
$ #Unnamed
# Nenosaukts
$ #MapRequest
# BlackboxWindow::mapRequestEvent() priekš 0x%lx\n
$ #UnmapNotify
# BlackboxWindow::unmapNotifyEvent() priekš 0x%lx\n
$ #ReparentNotify
# BlackboxWindow::unmapnotifyEvent: pārvecāka 0x%lx uz 0x%lx\n
