$set 9 #Window


$ #Creating
# BlackboxWindow::BlackboxWindow: 0x%lx を生成中\n
$ #XGetWindowAttributesFail
# BlackboxWindow::BlackboxWindow: XGetWindowAttributres 失敗\n
$ #CannotFindScreen
# BlackboxWindow::BlackboxWindow: ルートウィンドウ 0x%lx に対するスクリーンが\
みつかりません\n
$ #Unnamed
# 名前なし
$ #MapRequest
# BlackboxWindow::mapRequestEvent() (0x%lx に対して)\n
$ #UnmapNotify
# BlackboxWindow::unmapNotifyEvent() (0x%lx に対して)\n
$ #ReparentNotify
# BlackboxWindow::unmapnotifyEvent: reparent 0x%lx to 0x%lx\n
