$set 3 #Configmenu

$ #ConfigOptions
# Opciones de Configuración
$ #FocusModel
# Modelo del Foco
$ #WindowPlacement
# Ubicación de las Ventanas
$ #ImageDithering
# Simulación Ordenada de Colores en Imágenes
$ #OpaqueMove
# Movimiento Opaco de Ventanas
$ #FullMax
# Maximización Total
$ #FocusNew
# Enfocar Nuevas Ventanas
$ #FocusLast
# Enfocar la Ultima Ventana en el Escritorio
$ #DisableBindings
# Deshabilitar Enlaces con Bloq Despl
$ #ClickToFocus
# Click Para Enfocar
$ #SloppyFocus
# Foco Sigue Al Mouse
$ #AutoRaise
# Elevar Automáticamente
$ #ClickRaise
# Click Para Elevar
$ #SmartRows
# Ubicación Inteligente (Filas)
$ #SmartCols
# Ubicación Inteligente (Columnas)
$ #Cascade
# Ubicación en Cascada
$ #LeftRight
# Izquierda A Derecha
$ #RightLeft
# Derecha A Izquierda
$ #TopBottom
# De Arriba Abajo
$ #BottomTop
# De Abajo Arriba
