$set 14 #main

$ #RCRequiresArg
# kļūda: '-rc' nepieciešams arguments\n
$ #DISPLAYRequiresArg
# kļūda: '-display' nepieciešams arguments\n
$ #WarnDisplaySet
# brīdinājums: nevarēju iestatīt vides mainīgo 'DISPLAY'\n
$ #Usage
# Blackbox %s : (c) 2001 - 2002 Sean 'Shaleh' Perry\n\
  \t\t\t 1997 - 2000, 2002 Brad Hughes\n\n\
  -display <virkne>\t\tlietot displeja savienojumu.\n\
  -rc <virkne>\t\t\tlietot citu resersu failu.\n\
  -version\t\t\tparādīt versiju un iziet.\n\
  -help\t\t\t\tparādīt šo palīdzības tekstu un iziet.\n\n
$ #CompileOptions
# Kompilācijas laika opcijas:\n\
  Atkļūdošana\t\t\t%s\n\
  Veidne:\t\t\t%s\n\
  8bpp Pakāpes Tonēšana:\t%s\n\n
