$set 15 #Common

$ #Yes
# Ja
$ #No
# Nei

$ #DirectionTitle
# Retning
$ #DirectionHoriz
# Horisontal
$ #DirectionVert
# Vertikal

$ #AlwaysOnTop
# Alltid øverst

$ #PlacementTitle
# Plassering
$ #PlacementTopLeft
# Øverst til venstre
$ #PlacementCenterLeft
# Venstre i midten
$ #PlacementBottomLeft
# Nederst til venstre
$ #PlacementTopCenter
# Øverst i midten
$ #PlacementBottomCenter
# Nederst i midten
$ #PlacementTopRight
# Øverst til høyre
$ #PlacementCenterRight
# Høyre i midten
$ #PlacementBottomRight
# Nederst til høyre

$ #AutoHide
# Skjul automatisk
