$set 14 #main

$ #RCRequiresArg
# chyba: '-rc' vyžaduje argument\n
$ #DISPLAYRequiresArg
# chyba: '-display' vyžaduje argument\n
$ #WarnDisplaySet
# varovanie: nie je možné nastaviť premennú prostredia 'DISPLAY'\n
$ #Usage
# Blackbox %s : (c) 2001 - 2002 Sean 'Shaleh' Perry\n\
  \t\t\t 1997 - 2000, 2002 Brad Hughes\n\n\
  -display <string>\t\tpoužije sa pripojenie k zadanému displeju.\n\
  -rc <string>\t\t\tpoužije sa alternatívny konfiguračný súbor.\n\
  -version\t\t\tzobrazí verziu a skončí.\n\
  -help\t\t\t\tzobrazí túto nápovedu a skončí.\n\n
$ #CompileOptions
# Nastavenia v čase kompilácie:\n\
  Ladenie\t\t\t%s\n\
  Shape:\t\t\t%s\n\
  8bpp Ordered Dithering:\t%s\n\n
