$set 3 #Configmenu

$ #ConfigOptions
# Inställningar
$ #FocusModel
# Fokusmodell
$ #WindowPlacement
# Fönsterplacering
$ #ImageDithering
# Bilddithering
$ #OpaqueMove
# Ogenomskinlig fönsterförflyttning
$ #FullMax
# Full maximering
$ #FocusNew
# Fokus på nya fönster
$ #FocusLast
# Fokusera fönster vid skrivbordsbyte
$ #DisableBindings
# Gör så att bindningar inte fungerar med Scroll Lock
$ #ClickToFocus
# Klicka för fokus
$ #SloppyFocus
# Hafsig fokus
$ #AutoRaise
# Höj automatiskt
$ #ClickRaise
# Klicka för att höja
$ #SmartRows
# Smart placering (Rader)
$ #SmartCols
# Smart placering (Kolumner)
$ #Cascade
# Kaskadplacering
$ #LeftRight
# Från vänster
$ #RightLeft
# Från höger
$ #TopBottom
# Uppifrån
$ #BottomTop
# Nerifrån
