$set 1 #BaseDisplay

$ #XError
# %s:  X kļūda: %s(%d) opkodi %d/%d\n  resurss 0x%lx\n
$ #SignalCaught
# %s: saņēma signālu %d\n
$ #ShuttingDown
# beidzējos\n
$ #Aborting
# pārtraucu... izmetu pamatatmiņu\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: savienojums ar X serveri neizdevās.\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: nevarēju nomarķēt displeja savienojumu kā aizvērt-pie-izpildes\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): izņemu slikto logu no notikumu rindas\n
