$set 3 #Configmenu

$ #ConfigOptions
# Konfiguracja
$ #FocusModel
# Sposób aktywacji okien 
$ #WindowPlacement
# Rozmieszczanie okien
$ #ImageDithering
# Rastrowanie obrazu
$ #OpaqueMove
# Wyświetlaj zawartość przesuwanego okna
$ #FullMax
# Pełnoekranowa maksymalizacja
$ #FocusNew
# Ustawiaj nowe okna jako aktywne
$ #FocusLast
# Zmień aktywne okno przy zmianie pulpitów
$ #DisableBindings
# Scroll Lock blokuje modyfikatory
$ #ClickToFocus
# Uaktywnia kliknięcie
$ #SloppyFocus
# Uaktywnia kursor
$ #AutoRaise
# Podnoszenie automatyczne
$ #ClickRaise
# Podnoszenie po kliknięciu
$ #SmartRows
# Optymalne (Wiersze)
$ #SmartCols
# Optymalne (Kolumny)
$ #Cascade
# Kaskada
$ #LeftRight
# od Lewej do Prawej
$ #RightLeft
# od Prawej do Lewej
$ #TopBottom
# z Góry na Dół
$ #BottomTop
# z Dołu do Góry
