$set 15 #Common

$ #Yes
# Ja
$ #No
# Nej

$ #DirectionTitle
# Riktning
$ #DirectionHoriz
# Horisontell
$ #DirectionVert
# Vertikal

$ #AlwaysOnTop
# Alltid överst

$ #PlacementTitle
# Placering
$ #PlacementTopLeft
# Uppe till vänster
$ #PlacementCenterLeft
# Mitten till vänster
$ #PlacementBottomLeft
# Nere till vänster
$ #PlacementTopCenter
# Uppe i mitten
$ #PlacementBottomCenter
# Nere i mitten
$ #PlacementTopRight
# Uppe till höger
$ #PlacementCenterRight
# Mitten till höger
$ #PlacementBottomRight
# Nere till höger

$ #AutoHide
# Göm automatiskt
