$set 16 #bsetroot

$ #MustSpecify
# %s: hiba: kötelező megadni egy kapcsolót: -solid, -mod, -gradient\n
$ #Usage
# %s 2.0: (c) 1997-2000 Brad Hughes\n\n\
	  (c) 2001-2002 Sean 'Shaleh' Perry\n\n\
  -display <string>        használandó kijelző\n\
  -mod <x> <y>             modula minta\n\
  -foreground, -fg <color> modula előtér színe\n\
  -background, -bg <color> modula háttér színe\n\n\
  -gradient <texture>      gradiens textúrája\n\
  -from <color>            kezdőszín\n\
  -to <color>              befejezőszín\n\n\
  -solid <color>           egyszerű háttér\n\n\
  -help                    súgóképernyő mutatása és kilépés\n

