$set 15 #Common

$ #Yes
# はい
$ #No
# いいえ

$ #DirectionTitle
# 方向
$ #DirectionHoriz
# 水平
$ #DirectionVert
# 垂直

$ #AlwaysOnTop
# 常に最前面に

$ #PlacementTitle
# 配置
$ #PlacementTopLeft
# 左上
$ #PlacementCenterLeft
# 左中央
$ #PlacementBottomLeft
# 左下
$ #PlacementTopCenter
# 中央上
$ #PlacementBottomCenter
# 中央下
$ #PlacementTopRight
# 右上
$ #PlacementCenterRight
# 右中央
$ #PlacementBottomRight
# 右下

$ #AutoHide
# 自動的に隠す
