$set 5 #Image

$ #ErrorCreatingSolidPixmap
# BImage::render_solid: ピックスマップ生成エラー\n
$ #ErrorCreatingXImage
# BImage::renderXImage: XImage 生成エラー\n
$ #UnsupVisual
# BImage::renderXImage: 未サポートの視覚効果\n
$ #ErrorCreatingPixmap
# BImage::renderPixmap: ピックスマップ生成エラー\n
$ #InvalidColormapSize
# BImageControl::BImageControl: 不正なカラーマップサイズ %d (%d/%d/%d) - 減色中\n
$ #ErrorAllocatingColormap
# BImageControl::BImageControl: カラーマップ確保エラー\n
$ #ColorAllocFail
# BImageControl::BImageControl: カラー %d/%d/%d の確保に失敗\n
$ #PixmapRelease
# BImageControl::~BImageControl: ピックスマップキャッシュ -  %d ピックスマップを解放\n
$ #PixmapCacheLarge
# BImageControl::renderImage: キャッシュが大きすぎるので強制的に掃除\n
$ #ColorParseError
# BImageControl::getColor: カラー解析エラー: '%s'\n
$ #ColorAllocError
# BImageControl::getColor: カラー確保エラー: '%s'\n
