$set 16 #bsetroot

$ #MustSpecify
# %s: エラー: 次の中から一つを選択しなければなりません: -solid, -mod, -gradient\n
$ #Usage
# %s 2.0: (c) 1997-2000 Brad Hughes\n\n\
	  (c) 2001-2002 Sean 'Shaleh' Perry\n\n\
  -display <string>        ディスプレイを指定\n\
  -mod <x> <y>             組み合わせパターン\n\
  -foreground, -fg <color> 組み合わせ前景色\n\
  -background, -bg <color> 組み合わせ背景色\n\n\
  -gradient <texture>      グラデーションテクスチャ\n\
  -from <color>            グラデーション最初の色\n\
  -to <color>              グラデーション終りの色\n\
  -solid <color>           単色\n\n\
  -help                    このヘルプを表示し、終了\n

