$set 14 #main

$ #RCRequiresArg
# Hiba: '-rc' kapcsoló paramétere hiányzik\n
$ #DISPLAYRequiresArg
# Hiba: '-display' kapcsoló paramétere hiányzik\n
$ #WarnDisplaySet
# Figyelem: nem lehet a 'DISPLAY' környezeti változót beállítani\n
$ #Usage
# Blackbox %s : (c) 2001 - 2002 Sean 'Shaleh' Perry\n\
  \t\t\t 1997 - 2000, 2002 Brad Hughes\n\n\
  -display <string>\t\thasználandó kijelző\n\
  -rc <string>\t\t\tválasztható konfigurációs file.\n\
  -version\t\t\tverziószám kijelzése és kilépés.\n\
  -help\t\t\t\tsúgóképernyő mutatása és kilépés.\n\n
$ #CompileOptions
# Fordításnál állítható lehetőségek:\n\
  Debugging\t\t\t%s\n\
  Shape:\t\t\t%s\n\
  8bpp Ordered Dithering:\t%s\n\n
