$set 15 #Common

$ #Yes
# Sí
$ #No
# No

$ #DirectionTitle
# Dirección
$ #DirectionHoriz
# Horizontal
$ #DirectionVert
# Vertical

$ #AlwaysOnTop
# Siempre Encima

$ #PlacementTitle
# Ubicación
$ #PlacementTopLeft
# Arriba, Izquierda
$ #PlacementCenterLeft
# Centro, Izquierda
$ #PlacementBottomLeft
# Abajo, Izquierda
$ #PlacementTopCenter
# Arriba, Centro
$ #PlacementBottomCenter
# Abajo, Centro
$ #PlacementTopRight
# Arriba, Derecha
$ #PlacementCenterRight
# Centro, Derecha
$ #PlacementBottomRight
# Abajo, Derecha

$ #AutoHide
# Ocultar Automáticamente
