$set 15 #Common

$ #Yes
# Jā
$ #No
# Nē

$ #DirectionTitle
# Virziens
$ #DirectionHoriz
# Horizontāls
$ #DirectionVert
# Vertikāls

$ #AlwaysOnTop
# Vienmēr virspusē

$ #PlacementTitle
# Novietošana
$ #PlacementTopLeft
# Augšā pa Kreisi
$ #PlacementCenterLeft
# Centrā pa Kreisi
$ #PlacementBottomLeft
# Apakšā pa Kreisi
$ #PlacementTopCenter
# Augšā Centrā
$ #PlacementBottomCenter
# Apakšā Centrā
$ #PlacementTopRight
# Augšā pa Labi
$ #PlacementCenterRight
# Centrā pa Labi
$ #PlacementBottomRight
# Apakšā pa Labi
$ #AutoHide
# Auto paslēpt
