$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: wystąpił błąd podczas kontaktowania się z X serwerem.\n\
Inny menedżer okien obsługuje panel graficzny %s.\n
$ #ManagingScreen
# BScreen::BScreen: obsługuję ekran %d visual 0x%lx, głębia kolorów %d\n
$ #FontLoadFail
# BScreen::LoadStyle(): nie można załadować czcionki '%s'\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): nie można załadować domyślnej czcionki.\n
$ #EmptyMenuFile
# %s: plik menu jest pusty\n
$ #xterm
# xterm
$ #Restart
# Restart
$ #Exit
# Wyjście
$ #EXECError
# BScreen::parseMenuFile: [exec] błąd, brak etykiety menu i/lub komendy\n
$ #EXITError
# BScreen::parseMenuFile: [exit] błąd, brak etykiety menu\n
$ #STYLEError
# BScreen::parseMenuFile: [style] błąd, brak etykiety menu i/lub nazwy pliku\n
$ #CONFIGError
# BScreen::parseMenuFile: [config] błąd, brak etykiety menu\n
$ #INCLUDEError
# BScreen::parseMenuFile: [include] błąd, brak nazwy pliku\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: [include] błąd, '%s' nie jest zwykłym plikiem\n
$ #SUBMENUError
# BScreen::parseMenuFile: [submenu] błąd, brak etykiety menu\n
$ #RESTARTError
# BScreen::parseMenuFile: [restart] błąd, brak etykiety menu\n
$ #RECONFIGError
# BScreen::parseMenuFile: [reconfig] błąd, brak etykiety menu\n
$ #STYLESDIRError
# BScreen::parseMenuFile: [stylesdir/stylesmenu] błąd, brak nazwy katalogu\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: [stylesdir/stylesmenu] błąd, '%s' nie jest \
katalogiem\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: [stylesdir/stylesmenu] błąd, '%s' nie istnieje\n
$ #WORKSPACESError
# BScreen::parseMenuFile: [workspaces] błąd, brak etykiety menu\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# S: %4d x W: %4d

