$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: um erro occorreu enquanto consultava o servidor X.\n  \
existe outro window menager rodando no display. %s.\n
$ #ManagingScreen
# BScreen::BScreen: managing screen %d using visual 0x%lx, depth %d\n
$ #FontLoadFail
# BScreen::LoadStyle(): impossivel carregar fonte '%s'\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): impossivel carregar fonte padrão.\n
$ #EmptyMenuFile
# %s: arquivo de menu vazio\n
$ #xterm
# xterm
$ #Restart
# Reiniciar
$ #Exit
# Sair
$ #EXECError
# BScreen::parseMenuFile: [exec] erro, sem rótulo de menu e/ou comando definido\n
$ #EXITError
# BScreen::parseMenuFile: [exit] erro, sem rótulo de menu definido\n
$ #STYLEError
# BScreen::parseMenuFile: [style] erro, sem rótulo de menu e/ou nome do arquivo \
definido\n
$ #CONFIGError
# BScreen::parseMenuFile: [config] erro, sem rótulo de menu definido\n
$ #INCLUDEError
# BScreen::parseMenuFile: [include] erro, nome de arquivo não definido\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: [include] error, '%s' não é um arquivo regular\n
$ #SUBMENUError
# BScreen::parseMenuFile: [submenu] erro, sem rótulo de menu definido\n
$ #RESTARTError
# BScreen::parseMenuFile: [restart] erro, sem rótulo de menu definido\n
$ #RECONFIGError
# BScreen::parseMenuFile: [reconfig] erro, sem rótulo de menu definido\n
$ #STYLESDIRError
# BScreen::parseMenuFile: [stylesdir/stylesmenu] erro, sem diretorio difinido\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: [stylesdir/stylesmenu] erro, '%s' não é um \
diretorio\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: [stylesdir/stylesmenu] erro, '%s' não existe\n
$ #WORKSPACESError
# BScreen::parseMenuFile: [workspaces] erro, sem rótulo de menu definido\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# W: %4d x H: %4d

