$set 9 #Window


$ #Creating
# BlackboxWindow::BlackboxWindow: создается 0x%lx\n
$ #XGetWindowAttributesFail
# BlackboxWindow::BlackboxWindow: ошибка процесса XGetWindowAttributres\n
$ #CannotFindScreen
# BlackboxWindow::BlackboxWindow: не найден экран для корневого окна 0x%lx\n
$ #Unnamed
# Без имени
$ #MapRequest
# BlackboxWindow::mapRequestEvent() для 0x%lx\n
$ #UnmapNotify
# BlackboxWindow::unmapNotifyEvent() для 0x%lx\n
$ #ReparentNotify
# BlackboxWindow::reparentNotifyEvent: смена родителя для 0x%lx на 0x%lx\n
