$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: произошла ошибка при обращении к X серверу.\n  \
другой менеджер окон уже запущен на дисплее %s.\n
$ #ManagingScreen
# BScreen::BScreen: обслуживается экран %d, используемый тип визуализации 0x%lx, глубина цвета %d\n
$ #FontLoadFail
# BScreen::LoadStyle(): нет возможности загрузить шрифт '%s'\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): нет возможности загрузить шрифт по умолчанию.\n
$ #EmptyMenuFile
# %s: пустой файл меню\n
$ #xterm
# XTerm
$ #Restart
# Перезапуск
$ #Exit
# Выход
$ #EXECError
# BScreen::parseMenuFile: ошибка [exec], не определено название пункта меню и/или комманда\n
$ #EXITError
# BScreen::parseMenuFile: ошибка [exit], не определено название пункта меню\n
$ #STYLEError
# BScreen::parseMenuFile: ошибка [style], не определено название пункта меню и/или имя файла\n
$ #CONFIGError
# BScreen::parseMenuFile: ошибка [config], не определено название пункта меню\n
$ #INCLUDEError
# BScreen::parseMenuFile: ошибка [include], не определено имя файла\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: ошибка [include], '%s' не обычный файл\n
$ #SUBMENUError
# BScreen::parseMenuFile: ошибка [submenu], не определено название пункта меню\n
$ #RESTARTError
# BScreen::parseMenuFile: ошибка [restart], не определено название пункта меню\n
$ #RECONFIGError
# BScreen::parseMenuFile: ошибка [reconfig], не определено название пункта меню\n
$ #STYLESDIRError
# BScreen::parseMenuFile: ошибка [stylesdir/stylesmenu], не определено имя директории\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: ошибка [stylesdir/stylesmenu], '%s' не директория\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: ошибка [stylesdir/stylesmenu], '%s' не существует\n
$ #WORKSPACESError
# BScreen::parseMenuFile: ошибка [workspaces], не определено название пункта меню\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# W: %4d x H: %4d

