$set 5 #Image

$ #ErrorCreatingSolidPixmap
# BImage::render_solid: fel under skapande av pixmap\n
$ #ErrorCreatingXImage
# BImage::renderXImage: fel under skapande av XImage\n
$ #UnsupVisual
# BImage::renderXImage: ej stöd för färgdjup\n
$ #ErrorCreatingPixmap
# BImage::renderPixmap: fel under skapande av pixmap\n
$ #InvalidColormapSize
# BImageControl::BImageControl: ogiltig färgkarta storlek %d (%d/%d/%d) - reducerar\n
$ #ErrorAllocatingColormap
# BImageControl::BImageControl: fel under allokering av färgkarta\n
$ #ColorAllocFail
# BImageControl::BImageControl: misslyckades att allokera färg %d/%d/%d\n
$ #PixmapRelease
# BImageControl::~BImageControl: pixmap cache - släpper %d pixmappar\n
$ #PixmapCacheLarge
# BImageControl::renderImage: stor cache, tvingar upprensning\n
$ #ColorParseError
# BImageControl::getColor: färgfel: '%s'\n
$ #ColorAllocError
# BImageControl::getColor: färgallokeringsfel: '%s'\n
