$set 1 #BaseDisplay

$ #XError
# %s:  X chyba: %s(%d) operačné kódy %d/%d\n  zdroj 0x%lx\n
$ #SignalCaught
# %s: prijatý signál %d\n
$ #ShuttingDown
# ukončuje sa\n
$ #Aborting
# preruąuje sa... výpis obsahu pamäte (dumping core)\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: zlyhalo pripojenie k X serveru\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: nie je moľné označi» pripojenie k obrazovke ako close-on-exec\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): odstraňuje sa chybné okno z fronty udalostí\n
