$set 16 #bsetroot

$ #MustSpecify
# %s: Erreur: il faut préciser : -solid, -mod, -gradient\n
$ #Usage
# %s 2.0: (c) 1997-2000 Brad Hughes\n\n\
	  (c) 2001-2002 Sean 'Shaleh' Perry\n\n\
  -display <chaîne>        	connexion à l'affichage\n\
  -mod <x> <y>             	motif\n\
  -foreground, -fg <couleur> 	couleur d'avant plan\n\
  -background, -bg <couleur> 	couleur d'arrière plan\n\n\
  -gradient <texture>      	texture du dégradé\n\
  -from <couleur>            	couleur de départ du dégradé\n\
  -to <couleur>              	couleur d'arrivée du dégradé\n\n\
  -solid <couleur>           	couleur uniforme\n\n\
  -help                    	affiche ce texte d'aide et quitte\n

