$set 14 #main

$ #RCRequiresArg
# Fehler: '-rc' benötigt einen Parameter\n
$ #DISPLAYRequiresArg
# Fehler: '-display' benötigt einen Parameter\n
$ #WarnDisplaySet
# Achtung: konnte die Umgebungsvariable 'DISPLAY' nicht setzen\n
$ #Usage
# Blackbox %s : (c) 2001 - 2002 Sean 'Shaleh' Perry\n\
  \t\t\t 1997 - 2000, 2002 Brad Hughes\n\n\
  -display <string>\t\tDisplay Verbindung.\n\
  -rc <string>\t\t\talternative Konfigurationsdatei.\n\
  -version\t\t\tzeige Versionsnummer und Beendung.\n\
  -help\t\t\t\tzeige diesen Hilfstext und Beendung.\n\n
$ #CompileOptions
# Optionen zum Kompilationszeitpunkt:\n\
  Debugging\t\t\t%s\n\
  Shape:\t\t\t%s\n\
  8bpp Ordered Dithering:\t%s\n\n
