$set 5 #Image

$ #ErrorCreatingSolidPixmap
# BImage::render_solid: chyba pri vytváraní pixmapy\n
$ #ErrorCreatingXImage
# BImage::renderXImage: chyba pri vytváraní XImage\n
$ #UnsupVisual
# BImage::renderXImage: nepodporované zobrazenie\n
$ #ErrorCreatingPixmap
# BImage::renderPixmap: chyba pri vytváraní pixmapy\n
$ #InvalidColormapSize
# BImageControl::BImageControl: chybná veľkost colormapy %d (%d/%d/%d) - redukuje sa\n
$ #ErrorAllocatingColormap
# BImageControl::BImageControl: chyba pri alokovaní colormapy\n
$ #ColorAllocFail
# BImageControl::BImageControl: chyba pri alokovaní farby %d/%d/%d\n
$ #PixmapRelease
# BImageControl::~BImageControl: vyrovnávacia pamäť pre pixmapy - uvoľňuje sa %d pixmáp\n
$ #PixmapCacheLarge
# BImageControl::renderImage: vyrovnávacia pamäť je príliš veľká, vynútené vyprázdnenie\n
$ #ColorParseError
# BImageControl::getColor: chyba pri analýze farby: '%s'\n
$ #ColorAllocError
# BImageControl::getColor: chyba pri alokovaní farby: '%s'\n
