$set 1 #BaseDisplay

$ #XError
# %s:  Erreur de X %s(%d) opcodes %d/%d\n  ressource 0x%lx\n
$ #SignalCaught
# %s: signal %d capturé\n
$ #ShuttingDown
# arrêt du programme\n
$ #Aborting
# annulation... génération du fichier core\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: échec de la connexion au serveur X\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: impossible d'attribuer close-on-exec à l'affichage\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): suppression d'une mauvaise fenêtre de la queue des événements\n
