$set 3 #Configmenu

$ #ConfigOptions
# 설정옵션
$ #FocusModel
# 창 활성화 방식
$ #WindowPlacement
# 창 배치
$ #ImageDithering
# 이미지 디더링
$ #OpaqueMove
# 불투명 이동 
$ #FullMax
# 완전 최대화
$ #FocusNew
# 새 창 활성화
$ #FocusLast
# 작업창 변경시 창 활성화
$ #ClickToFocus
# 클릭 활성화
$ #SloppyFocus
# 슬라피 활성화
$ #AutoRaise
# 자동으로 가져오기
$ #ClickRaise
# 클릭으로 가져오기
$ #SmartRows
# 지능적 창배치 (열)
$ #SmartCols
# 지능정 창배치 (행)
$ #Cascade
# 계단식 창배치
$ #LeftRight
# 왼쪽에서 오른쪽으로
$ #RightLeft
# 오른쪽에서 왼쪽으로
$ #TopBottom
# 위에서 아래로
$ #BottomTop
# 아래에서 위로
