$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: hiba az X szerver kérdezésénél.\n \
Egy másik ablakkezelő már használja a %s képernyőt.\n
$ #ManagingScreen
# BScreen::BScreen: átvette az irányítást a %d képernyőn. visual: 0x%lx, színmélység: %d\n
$ #FontLoadFail
# BScreen::LoadStyle(): nem lehet a '%s' betűtípust betölteni\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): az alapértelmezett betűtípus betöltése nem lehetséges.\n
$ #EmptyMenuFile
# %s: üres menü file\n
$ #xterm
# terminál
$ #Restart
# Újraindítás
$ #Exit
# Kilépés
$ #EXECError
# BScreen::parseMenuFile: [exec] Hiba, nincs menü címke és/vagy parancs meghatározva\n
$ #EXITError
# BScreen::parseMenuFile: [exit] Hiba, nincs menü címke meghatározva\n
$ #STYLEError
# BScreen::parseMenuFile: [style] Hiba, nincs menü címke és/vagy file-név meghatározva\n
$ #CONFIGError
# BScreen::parseMenuFile: [config] Hiba, nincs menü címke meghatározva\n
$ #INCLUDEError
# BScreen::parseMenuFile: [include] Hiba, nincs file-név meghatározva\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: [include] Hiba, '%s' nem egy szabályos file\n
$ #SUBMENUError
# BScreen::parseMenuFile: [submenu] Hiba, nincs menü címke meghatározva\n
$ #RESTARTError
# BScreen::parseMenuFile: [restart] Hiba, nincs menü címke meghatározva\n
$ #RECONFIGError
# BScreen::parseMenuFile: [reconfig] Hiba, nincs menü címke meghatározva\n
$ #STYLESDIRError
# BScreen::parseMenuFile: [stylesdir/stylesmenu] Hiba, nincs meghatározva könyvtár\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: [stylesdir/stylesmenu] Hiba, '%s' nem könyvtár\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: [stylesdir/stylesmenu] Hiba, '%s' nem létezik\n
$ #WORKSPACESError
# BScreen::parseMenuFile: [workspaces] Hiba, nincs menü címke meghatározva\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# W: %4d x H: %4d

