$set 1 #BaseDisplay

$ #XError
# %s:  X 错误: %s(%d) opcodes %d/%d\n  resource 0x%lx\n
$ #SignalCaught
# %s: 捕获信号 %d \n
$ #ShuttingDown
# 关闭\n
$ #Aborting
# 正在退出... 存储核心数据\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: 连接 X 服务器失败.\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: couldn't mark display connection as close-on-exec\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): 正在从事件队列中删除坏窗口\n
