$set 1 #BaseDisplay

$ #XError
# %s:  X-fel: %s(%d) opkod %d/%d\n  resurs 0x%lx\n
$ #SignalCaught
# %s: signal %d fångad\n
$ #ShuttingDown
# stänger ner\n
$ #Aborting
# avbryter... dumpar kärna\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: anslutning till X server misslyckades.\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: kunde inte markera skärmanslutning som stäng-vid-exekvering\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): tar bort dåligt fönster från händelsekön\n
