$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: X サーバ問合せ中にエラー発生\n  \
他のウィンドウマネージャがディスプレイ %s で既に実行中です.\n
$ #ManagingScreen
# BScreen::BScreen: スクリーン %d を視覚効果 0x%lx, 深度 %d で制御中\n
$ #FontLoadFail
# BScreen::LoadStyle(): フォント '%s' を読み込めませんでした\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): 標準フォントを読み込めませんでした\n
$ #EmptyMenuFile
# %s: 空のメニューファイル\n
$ #xterm
# xterm
$ #Restart
# 再起動
$ #Exit
# 終了
$ #EXECError
# BScreen::parseMenuFile: [exec] エラー、メニューラベル(コマンド)未定義\n
$ #EXITError
# BScreen::parseMenuFile: [exit] エラー、メニューラベル未定義\n
$ #STYLEError
# BScreen::parseMenuFile: [style] エラー、メニューラベル(ファイル名)未定義\n
$ #CONFIGError
# BScreen::parseMenuFile: [config] エラー、メニューラベル未定義\n
$ #INCLUDEError
# BScreen::parseMenuFile: [include] エラー、ファイル名未定義\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: [include] エラー、'%s' は正規ファイルではありません\n
$ #SUBMENUError
# BScreen::parseMenuFile: [submenu] エラー、メニューラベル未定義\n
$ #RESTARTError
# BScreen::parseMenuFile: [restart] エラー、メニューラベル未定義\n
$ #RECONFIGError
# BScreen::parseMenuFile: [reconfig] エラー、メニューラベル未定義\n
$ #STYLESDIRError
# BScreen::parseMenuFile: [stylesdir/stylesmenu] エラー、ディレクトリ名未定義\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: [stylesdir/stylesmenu] エラー、'%s' はディレクトリでは\
ありません\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: [stylesdir/stylesmenu] エラー、'%s' は存在しません\n
$ #WORKSPACESError
# BScreen::parseMenuFile: [workspaces] エラー、メニューラベル未定義\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# 幅: %4d x 高さ: %4d

