$set 14 #main

$ #RCRequiresArg
# erro: '-rc' requer um argumento\n
$ #DISPLAYRequiresArg
# erro: '-display' requer um argumento\n
$ #WarnDisplaySet
# aviso: não foi possivel setar a variavel de ambiente 'DISPLAY'\n
$ #Usage
# Blackbox %s : (c) 2001 - 2002 Sean 'Shaleh' Perry\n\
  \t\t\t 1997 - 2000, 2002 Brad Hughes\n\n\
  -display <string>\t\tusar conexão com o display.\n\
  -rc <string>\t\t\tusar arquivo alternativo de recursos.\n\
  -version\t\t\texibe a versao e sair.\n\
  -help\t\t\t\texibe este texto de ajuda e sair.\n\n
$ #CompileOptions
# Opções em tempo de compilação:\n\
  Depuração:\t\t\t%s\n\
  Forma:\t\t\t%s\n\
  Simulação Ordenada de Cores em 8bpp:\t%s\n\n
