$set 1 #BaseDisplay

$ #XError
# %s:  X hiba: %s(%d) opcodes %d/%d\n  resource 0x%lx\n
$ #SignalCaught
# %s: %d szignált kapott\n
$ #ShuttingDown
# Leállás\n
$ #Aborting
# Programhiba... dumping core\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: nem sikerült kapcsolódni az X szerverhez.\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: az összeköttetés a képernyővel nem 'close-on-exit'\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): helytelen ablak kivétele a 'Event Queue'-ból\n
