$set 3 #Configmenu

$ #ConfigOptions
# Options de configuration
$ #FocusModel
# Mode d'attribution du clavier
$ #WindowPlacement
# Disposition des fenêtres
$ #ImageDithering
# Lissage des images
$ #OpaqueMove
# Déplacement opaque des fenêtres
$ #FullMax
# Maximisation complète
$ #FocusNew
# Attribuer le clavier à la nouvelle fenêtre
$ #FocusLast
# Attribuer le clavier en changeant d'espace de travail
$ #DisableBindings
# Désactiver les modificateurs avec Vérouillage Défilement
$ #ClickToFocus
# Cliquer pour obtenir le clavier
$ #SloppyFocus
# Attribution du clavier souple
$ #AutoRaise
# Premier plan automatique
$ #ClickRaise
# Cliquer pour mettre au premier plan
$ #SmartRows
# Disposition futée (Lignes)
$ #SmartCols
# Disposition futée (Colonnes)
$ #Cascade
# Disposition en cascade
$ #LeftRight
# De gauche à droite
$ #RightLeft
# De droite à gauche
$ #TopBottom
# De haut en bas
$ #BottomTop
# De bas en haut
