$set 1 #BaseDisplay

$ #XError
# %s:  błąd X: %s(%d) opcodes %d/%d\n  resource 0x%lx\n
$ #SignalCaught
# %s: złapano sygnał %d\n
$ #ShuttingDown
# zamykam\n
$ #Aborting
# przerywam... zrzucam core\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: Nie można połączyć się z X serwerem.\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: Nie można ustawić połączenia jako 'close-on-exec'\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): Usuwam złe okno z kolejki zdarzeń\n
