$set 3 #Configmenu

$ #ConfigOptions
# Konfigurācijas Opcijas
$ #FocusModel
# Fokusēšanās Modelis
$ #WindowPlacement
# Loga Novietojums
$ #ImageDithering
# Attēla Tonēšana
$ #OpaqueMove
# Necaurredzama Loga Pārvietošana
$ #FullMax
# Pilna Maksimizācija
$ #FocusNew
# Fokusēt Jaunos Logus
$ #DisableBindings
# Deaktivizēt Sasaistes ar Ritslēga taustiņu
$ #FocusLast
# Fokusēt Logu pie Darba vietas maiņas
$ #ClickToFocus
# Klikšķis, lai Fokusētu
$ #SloppyFocus
# Nevīžīga Fokusēšana
$ #AutoRaise
# Auto Pacelšana
$ #ClickRaise
# Klikšķis Paceļ
$ #SmartRows
# Gudrā Novietošana (Rindās)
$ #SmartCols
# Gudrā Novietošana (Kolonnās)
$ #Cascade
# Kaskādes Novietošana
$ #LeftRight
# No Kreisās un Labo
$ #RightLeft
# No Labās uz Kreiso
$ #TopBottom
# No Augšas uz Leju
$ #BottomTop
# No Lejas uz Augšu
