$set 1 #BaseDisplay

$ #XError
# %s:  X error: %s(%d) código de operación %d/%d\n  recurso 0x%lx\n
$ #SignalCaught
# %s: señal %d atrapada\n
$ #ShuttingDown
# apagando\n
$ #Aborting
# abortando... grabando 'core'\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: falló la conexión al servidor de X.\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: no pude marcar la conexión al display como close-on-exec\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): quitando ventana errónea de la lista de eventos\n
