$set 3 #Configmenu

$ #ConfigOptions
# 設定オプション
$ #FocusModel
# フォーカスモデル
$ #WindowPlacement
# ウィンドウ配置
$ #ImageDithering
# 画像ディザ
$ #OpaqueMove
# 不透明にしてウィンドウを移動
$ #FullMax
# 完全最大化
$ #FocusNew
# 新規ウィンドウにフォーカス
$ #FocusLast
# ワークスペース移動時にウィンドウにフォーカス
$ #DisableBindings
# スクロールロック時にショートカットを無効に
$ #ClickToFocus
# クリックでフォーカス
$ #SloppyFocus
# Sloppy Focus
$ #AutoRaise
# 自動的に前面に
$ #ClickRaise
# クリックで前面に
$ #SmartRows
# 賢く配置 (行)
$ #SmartCols
# 賢く配置 (列)
$ #Cascade
# ずらして配置
$ #LeftRight
# 左から右へ
$ #RightLeft
# 右から左へ
$ #TopBottom
# 上から下へ
$ #BottomTop
# 下から上へ
