$set 3 #Configmenu

$ #ConfigOptions
# Indstillinger
$ #FocusModel
# Fokus Model
$ #WindowPlacement
# Vindueplacering
$ #ImageDithering
# Billede Dithering
$ #OpaqueMove
# Uklar vinduesflytning
$ #FullMax
# Fuld maximering
$ #FocusNew
# Fokus nyt vindue
$ #FocusLast
# Fokus vindue ved skrivebords ændring
$ #DisableBindings
# Deaktiver tastebindinger med Scroll Lock
$ #ClickToFocus
# Klik for fokus
$ #SloppyFocus
# Dovent fokus
$ #AutoRaise
# Hæv automatisk
$ #ClickRaise
# Klik for at hæve
$ #SmartRows
# Intelligent placering (Rækker)
$ #SmartCols
# Intelligent placering (Søjler)
$ #Cascade
# Kaskade placering
$ #LeftRight
# Fra venstre mod højre
$ #RightLeft
# Fra højre mod venstre
$ #TopBottom
# Oppefra
$ #BottomTop
# Nedefra
