$set 3 #Configmenu

$ #ConfigOptions
# Configurações
$ #FocusModel
# Modelo de Foco
$ #WindowPlacement
# Posicionamento das Janelas
$ #ImageDithering
# Simulação de Cores em Imagens
$ #OpaqueMove
# Movimento Opaco de Janelas
$ #FullMax
# Maximização Total
$ #FocusNew
# Foco em Novas Janelas
$ #FocusLast
# Foco na Ultima Janela Da Area de Trabalho
$ #DisableBindings
# Disabilitar Ações com Scroll Lock
$ #ClickToFocus
# Clicar para Focar
$ #SloppyFocus
# Apontar para Focar
$ #AutoRaise
# Auto-sobrepor
$ #ClickRaise
# Click Raise
$ #SmartRows
# Organizar Rapido (Linha)
$ #SmartCols
# Organizar Rapido (Coluna)
$ #Cascade
# Organizar em Cascata
$ #LeftRight
# Esquerda para Direita
$ #RightLeft
# Direita para Esquerda
$ #TopBottom
# Cima para Baixo
$ #BottomTop
# Baixo para Cima
