$set 10 #Windowmenu

$ #SendTo
# Umiestni» na ...
$ #Shade
# Zrolova»
$ #Iconify
# Zmeni» na ikonu
$ #Maximize
# Maximalizova»
$ #Raise
# Presunú» do popredia
$ #Lower
# Presunú» do pozadia
$ #Stick
# Prilepi»
$ #KillClient
# Ukonči» klienta
$ #Close
# Zavrie»
