$set 10 #Windowmenu

$ #SendTo
# Отправить на ...
$ #Shade
# Свернуть в заголовок
$ #Iconify
# Свернуть в иконку
$ #Maximize
# Максимизировать
$ #Raise
# Поднять наверх
$ #Lower
# Опустить вниз
$ #Stick
# Приклеить
$ #KillClient
# Убить клиентское приложение
$ #Close
# Закрыть
