$set 16 #bsetroot

$ #MustSpecify
# %s: помилка: необхідно задати один з наступних ключів: -solid, -mod, -gradient\n
$ #Usage
# %s 2.0: (c) 1997-2000 Brad Hughes\n\n\
	  (c) 2001-2002 Sean 'Shaleh' Perry\n\n\
  -display <string>        обслуговувати вказаний дисплей\n\
  -mod <x> <y>             макет клітки\n\
  -foreground, -fg <color> колір переднього плану клітки\n\
  -background, -bg <color> колір тла клітки\n\n\
  -gradient <texture>      градієнт\n\
  -from <color>            початковий колір градієнта\n\
  -to <color>              кінцевий колір градієнта\n\n\
  -solid <color>           суцільний колір\n\n\
  -help                    вивести цю підказку і вийти\n

