$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: pri dopytovaní X servera nastala chyba.\n  \
na displeji je práve spustený ďalší správca okien %s.\n
$ #ManagingScreen
# BScreen::BScreen: riadiaca obrazovka %d používa zobrazenie 0x%lx, hĺbka %d\n
$ #FontLoadFail
# BScreen::LoadStyle(): nie je možné nahrať font '%s'\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): nie je možné nahrať predvolený font.\n
$ #EmptyMenuFile
# %s: prázdny menu súbor\n
$ #xterm
# xterm
$ #Restart
# Reštart
$ #Exit
# Koniec
$ #EXECError
# BScreen::parseMenuFile: [exec] chyba, nedefinovaný titulok menu a/alebo príkaz\n
$ #EXITError
# BScreen::parseMenuFile: [exit] chyba, nedefinovaný titulok menu\n
$ #STYLEError
# BScreen::parseMenuFile: [style] chyba, nedefinovaný titulok menu a/alebo názov súboru\n
$ #CONFIGError
# BScreen::parseMenuFile: [config] chyba, nedefinovaný titulok menu\n
$ #INCLUDEError
# BScreen::parseMenuFile: [include] chyba, nedefinovaný názov súboru\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: [include] chyba, '%s' nie je obyčajný súbor\n
$ #SUBMENUError
# BScreen::parseMenuFile: [submenu] chyba, nedefinovaný titulok menu\n
$ #RESTARTError
# BScreen::parseMenuFile: [restart] chyba, nedefinovaný titulok menu\n
$ #RECONFIGError
# BScreen::parseMenuFile: [reconfig] chyba, nedefinovaný titulok menu\n
$ #STYLESDIRError
# BScreen::parseMenuFile: [stylesdir/stylesmenu] chyba, nedefinovaný adresár\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: [stylesdir/stylesmenu] chyba, '%s' nie je adresár\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: [stylesdir/stylesmenu] chyba, '%s' neexistuje\n
$ #WORKSPACESError
# BScreen::parseMenuFile: [workspaces] chyba, nedefinovaný titulok menu\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# W: %4d x H: %4d

