$set 14 #main

$ #RCRequiresArg
# błąd: opcja '-rc' wymaga podania argumentu\n
$ #DISPLAYRequiresArg
# błąd: opcja '-display' wymaga podania argumentu\n
$ #WarnDisplaySet
# ostrzeżenie: nie można ustawić zmiennej środowiskowej 'DISPLAY'\n
$ #Usage
# Blackbox %s : (c) 2001 - 2002 Sean 'Shaleh' Perry\n\
  \t\t\t 1997 - 2000, 2002 Brad Hughes\n\n\
  -display <string>\t\tpołącz z podanym panelem graficznym.\n\
  -rc <string>\t\t\tużyj innego pliku konfiguracyjnego.\n\
  -version\t\t\twyświetl wersję i zakończ działanie programu.\n\
  -help\t\t\t\twyświetl ten komunikat.\n\n
$ #CompileOptions
# Opcje użyte podczas kompilacji:\n\
  Debugging\t\t\t%s\n\
  Shape:\t\t\t%s\n\
  8bpp Ordered Dithering:\t%s\n\n
