$set 15 #Common

$ #Yes
# Так
$ #No
# Ні

$ #DirectionTitle
# Напрямок
$ #DirectionHoriz
# Горизонтально
$ #DirectionVert
# Вертикально

$ #AlwaysOnTop
# Завжди нагорі

$ #PlacementTitle
# Місце розташування
$ #PlacementTopLeft
# Угорі і ліворуч 
$ #PlacementCenterLeft
# По центру і ліворуч 
$ #PlacementBottomLeft
# Унизу і ліворуч 
$ #PlacementTopCenter
# Зверху і по центру
$ #PlacementBottomCenter
# Знизу і по центру
$ #PlacementTopRight
# Зверху і праворуч
$ #PlacementCenterRight
# По центру і праворуч
$ #PlacementBottomRight
# Унизу і праворуч

$ #AutoHide
# Автоматично ховати
