$set 16 #bsetroot

$ #MustSpecify
# %s: błąd: należy użyć jednej z opcji: -solid, -mod, -gradient\n
$ #Usage
# %s 2.0: (c) 1997-2000 Brad Hughes\n\n\
	  (c) 2001-2002 Sean 'Shaleh' Perry\n\n\
  -display <string>        połącz z podanym panelem graficznym\n\
  -mod <x> <y>             współczynniki siatki typu modula\n\
  -foreground, -fg <color> kolor siatki\n\
  -background, -bg <color> kolor tła siatki\n\n\
  -gradient <texture>      tekstura gradientowa\n\
  -from <color>            początkowy kolor gradientu\n\
  -to <color>              końcowy kolor gradientu\n\n\
  -solid <color>           jednolite wypełnienie podanym kolorem\n\n\
  -help                    wyświetla ten komunikat\n

