$set 14 #main

$ #RCRequiresArg
# errore: '-rc' richiede un argomento\n
$ #DISPLAYRequiresArg
# errore: '-display' richiede an argomento\n
$ #WarnDisplaySet
# attenzione: impossibile impostare la variabile d'ambiente 'DISPLAY'\n
$ #Usage
# Blackbox %s : (c) 2001 - 2002 Sean 'Shaleh' Perry\n\
  \t\t\t 1997 - 2000, 2002 Brad Hughes\n\n\
  -display <string>\t\tusa connessione display.\n\
  -rc <string>\t\t\tusa un file risorsa alternativo.\n\
  -version\t\t\tmostra la versione e esci.\n\
  -help\t\t\t\tmostra questo aiuto e esci.\n\n
$ #CompileOptions
# Opzioni in fase di compilazione:\n\
  Debugging\t\t\t%s\n\
  Shape:\t\t\t%s\n\
  8bpp Ordered Dithering:\t%s\n\n
