$set 16 #bsetroot

$ #MustSpecify
# %s: chyba: musíte špecifikovať jeden z parametrov: -solid, -mod, -gradient\n
$ #Usage
# %s 2.0: (c) 1997-2000 Brad Hughes\n\n\
	  (c) 2001-2002 Sean 'Shaleh' Perry\n\n\
  -display <string>        pripojenie k obrazovke\n\
  -mod <x> <y>             modul vzorky\n\
  -foreground, -fg <color> modul farby popredia\n\
  -background, -bg <color> modul farby pozadia\n\n\
  -gradient <texture>      textúra farebného prechodu\n\
  -from <color>            počiatočná farba vo farebnom prechode\n\
  -to <color>              konečná farba vo farebnom prechode\n\n\
  -solid <color>           jednoliata farba\n\n\
  -help                    vypíše túto nápovedu a skončí\n

