$set 16 #bsetroot

$ #MustSpecify
# %s: fel: måste specifiera en av: -solid, -mod, -gradient\n
$ #Usage
# %s 2.0: (c) 1997-2000 Brad Hughes\n\n\
	  (c) 2001-2002 Sean 'Shaleh' Perry\n\n\
  -display <string>        skärmanslutning\n\
  -mod <x> <y>             modulamönster\n\
  -foreground, -fg <color> förgrundsfärg för modulamönster\n\
  -background, -bg <color> bakgrundsfärg för modulamönster\n\n\
  -gradient <texture>      texturlutning\n\
  -from <color>            startfärg\n\
  -to <color>              slutfärg\n\n\
  -solid <color>           solid färg\n\n\
  -help                    visa denna hjälptext och avsluta\n

