$set 10 #Windowmenu

$ #SendTo
# Wyślij na ...
$ #Shade
# Zwiń
$ #Iconify
# Ikonizuj
$ #Maximize
# Maksymalizuj
$ #Raise
# Na wierzch 
$ #Lower
# Pod spód
$ #Stick
# Przyklej
$ #KillClient
# Zabij
$ #Close
# Zamknij
