$set 5 #Image

$ #ErrorCreatingSolidPixmap
# BImage::render_solid: wystąpił błąd podczas tworzenia pixmapy\n
$ #ErrorCreatingXImage
# BImage::renderXImage: wystąpił błąd podczas tworzenia obiektu XImage\n
$ #UnsupVisual
# BImage::renderXImage: brak akceptowanego sposobu obsługi kolorów\n
$ #ErrorCreatingPixmap
# BImage::renderPixmap: wystąpił błąd podczas tworzenia pixmapy\n
$ #InvalidColormapSize
# BImageControl::BImageControl: zły rozmiar mapy kolorów %d (%d/%d/%d) - \
redukuje\n
$ #ErrorAllocatingColormap
# BImageControl::BImageControl: błąd alokacji mapy kolorów\n
$ #ColorAllocFail
# BImageControl::BImageControl: alokacja koloru %d/%d/%d nieudana\n
$ #PixmapRelease
# BImageControl::BImageControl: pixmap cache - zwalniam %d pixmap(y)\n
$ #PixmapCacheLarge
# BImageControl::renderImage: duża pamięć cache, wymuszam czyszczenie\n
$ #ColorParseError
# BImageControl::getColor: wystąpił błąd podczas tłumaczenia koloru: '%s'\n
$ #ColorAllocError
# BImageControl::getColor: wystąpił błąd podczas alokacji koloru: '%s'\n
