$set 16 #bsetroot

$ #MustSpecify
# %s: kļūda: jānosaka viens no: -solid, -mod, -gradient\n
$ #Usage
# %s 2.0: (c) 1997-2000 Brad Hughes\n\n\
  -display <virkne>        displeja savienojums\n\
  -mod <x> <y>             moduļa raksts\n\
  -foreground, -fg <krāsa> moduļa priekšplāna krāsa\n\
  -background, -bg <krāsa> moduļa fona krāsa\n\n\
  -gradient <tekstūra>     gradienta tekstūra\n\
  -from <krāsa>            pārejas sākuma krāsa\n\
  -to <krāsa>              pārejas beigu krāsa\n\n\
  -solid <krāsa>           viendabīga krāsa\n\n\
  -help                    parādīt šo palīdzību un iziet\n

