$set 9 #Window


$ #Creating
# BlackboxWindow::BlackboxWindow: 正在创建 0x%lx\n
$ #XGetWindowAttributesFail
# BlackboxWindow::BlackboxWindow: XGetWindowAttributres 失败\n
$ #CannotFindScreen
# BlackboxWindow::BlackboxWindow: cannot find screen for root window 0x%lx\n
$ #Unnamed
# 无命名
$ #MapRequest
# BlackboxWindow::mapRequestEvent() for 0x%lx\n
$ #UnmapNotify
# BlackboxWindow::unmapNotifyEvent() for 0x%lx\n
$ #ReparentNotify
# BlackboxWindow::reparentNotifyEvent: reparent 0x%lx to 0x%lx\n
