$set 1 #BaseDisplay

$ #XError
# %s:  ошибка X сервера: %s(%d) значения %d/%d\n ресурс 0x%lx\n
$ #SignalCaught
# %s: получен сигнал %d \n
$ #ShuttingDown
# shutting down\n
$ #Aborting
# Отмена... создается дамп core\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: ошибка соединения с X сервером.\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: нет возможности пометить активный дисплей как "закрытый-на-исполнение"\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): плохое окно удаляется из очереди событий\n
