$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: Erreur survenue pendant une requête adressée au serveur X.\n  \
Un autre gestionnaire de fenêtres est déjà lancé %s.\n
$ #ManagingScreen
# BScreen::BScreen: Gestion de l'écran %d avec le mode visuel 0x%lx, profondeur %d\n
$ #FontLoadFail
# BScreen::LoadStyle(): Impossible de charger la police de caractères '%s'\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): Impossible de charger la police de caractères initiale.\n
$ #EmptyMenuFile
# %s: Fichier de menu vide\n
$ #xterm
# xterm
$ #Restart
# Redémarrer
$ #Exit
# Sortie
$ #EXECError
# BScreen::parseMenuFile: [exec] Erreur, pas de libellé de menu et/ou de commande definie\n
$ #EXITError
# BScreen::parseMenuFile: [exit] Erreur, aucun libellé de menu défini\n
$ #STYLEError
# BScreen::parseMenuFile: [style] Erreur, pas libellé de menu et/ou de nom de fichier\
defini\n
$ #CONFIGError
# BScreen::parseMenuFile: [config] Erreur, aucun libellé de menu défini\n
$ #INCLUDEError
# BScreen::parseMenuFile: [include] Erreur, aucun nom de fichier défini\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: [include] Erreur, '%s' est un fichier invalide\n
$ #SUBMENUError
# BScreen::parseMenuFile: [submenu] Erreur, aucun libellé de menu défini\n
$ #RESTARTError
# BScreen::parseMenuFile: [restart] Erreur, aucun libellé de menu défini\n
$ #RECONFIGError
# BScreen::parseMenuFile: [reconfig] Erreur, aucun libellé de menu défini\n
$ #STYLESDIRError
# BScreen::parseMenuFile: [stylesdir/stylesmenu] Erreur, aucun répertoire défini\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: [stylesdir/stylesmenu] Erreur, '%s' est un répertoire \
invalide\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: [stylesdir/stylesmenu] Erreur, '%s' est inexistant\n
$ #WORKSPACESError
# BScreen::parseMenuFile: [workspaces] Erreur, aucun libellé de menu défini\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# L: %4d x H: %4d

