$set 3 #Configmenu

$ #ConfigOptions
# Beállítások
$ #FocusModel
# Fókusz modell
$ #WindowPlacement
# Ablakok elhelyezése
$ #ImageDithering
# Image dithering
$ #OpaqueMove
# Átlátszó ablakok mozgatás közben
$ #FullMax
# Teljesképernyős maximalizálás
$ #FocusNew
# Új ablak kapja a fókuszt
$ #FocusLast
# Munkaasztal váltása után fókusz az utolsó ablakon
$ #DisableBindings
# Scroll Lock-nál nem figyeli a billentyűzetet
$ #ClickToFocus
# Kattintás fókuszhoz
$ #SloppyFocus
# A fókusz követi az egeret
$ #AutoRaise
# Autómatikus előreugró ablakok
$ #ClickRaise
# Kattintással az előtérbe
$ #SmartRows
# Optimális ablakrendezés (sorok)
$ #SmartCols
# Optimális ablakrendezés (oszlopok)
$ #Cascade
# Egymásracsúsztatott elrendezés
$ #LeftRight
# Jobbról balra
$ #RightLeft
# Balról jobbra
$ #TopBottom
# Fentről lefele
$ #BottomTop
# Lentről felfele
