$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: сталася помилка при звертанні до X сервера.\n  \
Інший менеджер вікон вже запущений на дисплеї %s.\n
$ #ManagingScreen
# BScreen::BScreen: обслуговується екран %d, який використовує тип візуалізації 0x%lx, глибина кольору %d\n
$ #FontLoadFail
# BScreen::LoadStyle(): нема можливості завантажити шрифт "%s"\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): нема можливості завантажити типовий шрифт.\n
$ #EmptyMenuFile
# %s: порожній файл меню\n
$ #xterm
# Xterm
$ #Restart
# Перезапуск
$ #Exit
# Вихід
$ #EXECError
# BScreen::parseMenuFile: помилка [exec], не вказана назва пункту меню і/або назва команди\n
$ #EXITError
# BScreen::parseMenuFile: помилка [exit], не вказана назва пункту меню\n
$ #STYLEError
# BScreen::parseMenuFile: помилка [style], не вказана назва пункту меню і/або ім'я файлу\n
$ #CONFIGError
# BScreen::parseMenuFile: помилка [config], не вказана назва пункту меню\n
$ #INCLUDEError
# BScreen::parseMenuFile: помилка [include], не вказано ім'я файлу\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: помилка [include], "%s" не звичайний файл\n
$ #SUBMENUError
# BScreen::parseMenuFile: помилка [submenu], не вказана назва пункту меню\n
$ #RESTARTError
# BScreen::parseMenuFile: помилка [restart], не вказана назва пункту меню\n
$ #RECONFIGError
# BScreen::parseMenuFile: помилка [reconfig], не вказана назва пункту меню\n
$ #STYLESDIRError
# BScreen::parseMenuFile: помилка [stylesdir/stylesmenu], не вказано ім'я каталогу\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: помилка [stylesdir/stylesmenu], "%s" не каталог\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: помилка [stylesdir/stylesmenu], "%s" не існує\n
$ #WORKSPACESError
# BScreen::parseMenuFile: помилка [workspaces], не вказана назва пункту меню\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# W: %4d x H: %4d

