$set 5 #Image

$ #ErrorCreatingSolidPixmap
# BImage::render_solid: hiba a kép készítésekor\n
$ #ErrorCreatingXImage
# BImage::renderXImage: hiba az XImage készítésekor\n
$ #UnsupVisual
# BImage::renderXImage: nem támogatott visual\n
$ #ErrorCreatingPixmap
# BImage::renderPixmap: hiba a pixmap készítésekor\n
$ #InvalidColormapSize
# BImageControl::BImageControl: helytelen colormap méret %d (%d/%d/%d) - csökkentés\n
$ #ErrorAllocatingColormap
# BImageControl::BImageControl: hiba a colormap lefoglalásánál\n
$ #ColorAllocFail
# BImageControl::BImageControl: hiba a %d/%d/%d szín lefoglalásánál\n
$ #PixmapRelease
# BImageControl::~BImageControl: pixmap cache - %d pixmaps felszabadítása\n
$ #PixmapCacheLarge
# BImageControl::renderImage: a cache túl nagy, ürítés\n
$ #ColorParseError
# BImageControl::getColor: hiba a szín elemzésénél: '%s'\n
$ #ColorAllocError
# BImageControl::getColor: hiba a szín lefoglalásánál: '%s'\n
