$set 1 #BaseDisplay

$ #XError
# %s:  X エラー: %s(%d) opcodes %d/%d\n  resource 0x%lx\n
$ #SignalCaught
# %s: シグナル %d 発生\n
$ #ShuttingDown
# 終了\n
$ #Aborting
# 中止 ... コアダンプ中.\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: X サーバへの接続に失敗\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: ディスプレイ接続に実行時クローズフラグをたてることができませんでした\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): イベントキューから不正なウィンドウを消します\n
