$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: atgadījās kļūda izvaicājot X-serveri.\n  \
jau darbojas cits logu pārvaldnieks uz displeja %s.\n
$ #ManagingScreen
# BScreen::BScreen: parvaldu ekrānu %d, lietojot vizuālo 0x%lx, dziļums %d\n
$ #FontLoadFail
# BScreen::LoadStyle(): nevarēju ielādēt fontu '%s'\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): nevarēju ielādēt noklusēto fontu.\n
$ #EmptyMenuFile
# %s: tukšs izvēlnes fails\n
$ #xterm
# xterm
$ #Restart
# Restartēt
$ #Exit
# Iziet
$ #EXECError
# BScreen::parseMenuFile: [exec] kļūda, nav definēta izvēlnes iezīme un/vai komanda\n
$ #EXITError
# BScreen::parseMenuFile: [exit] kļūda, nav definēta izvēlnes iezīme\n
$ #STYLEError
# BScreen::parseMenuFile: [style] kļūda, nav definēta izvēlnes iezīme un/vai \
faila nosaukums\n
$ #CONFIGError
# BScreen::parseMenuFile: [config] kļūda, nav definēta izvēlnes iezīme\n
$ #INCLUDEError
# BScreen::parseMenuFile: [include] kļūda, nav definēts faila nosaukums\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: [include] kļūda, '%s' nav parasts fails\n
$ #SUBMENUError
# BScreen::parseMenuFile: [submenu] kļūda, nav definēta izvēlnes iezīme\n
$ #RESTARTError
# BScreen::parseMenuFile: [restart] kļūda, nav definēta izvēlnes iezīme\n
$ #RECONFIGError
# BScreen::parseMenuFile: [reconfig] kļūda, nav definēta izvēlnes iezīme\n
$ #STYLESDIRError
# BScreen::parseMenuFile: [stylesdir/stylesmenu] kļūda, nav definēta direktorija\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: [stylesdir/stylesmenu] kļūda, '%s' nav \
direktorija\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: [stylesdir/stylesmenu] kļūda, '%s' neeksistē\n
$ #WORKSPACESError
# BScreen::parseMenuFile: [workspaces] kļūda, nav definēta izvēlnes iezīme\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# P: %4d x A: %4d

