$set 3 #Configmenu

$ #ConfigOptions
# Конфігурація
$ #FocusModel
# Модель фокусування
$ #WindowPlacement
# Місцезнаходження вікна
$ #ImageDithering
# Емулювати кольори у зображеннях
$ #OpaqueMove
# Перенесення заповнених вікон
$ #FullMax
# Розгортати на весь екран
$ #FocusNew
# Давати фокус новим вікнам
$ #FocusLast
# Давати фокус при зміні робочого місця
$ #ClickToFocus
# Клацнути для отримання фокусу
$ #SloppyFocus
# Фокус іде за мишкою
$ #AutoRaise
# Автопіднімання
$ #SmartRows
# "Розумне" розташування (горизонтально)
$ #SmartCols
# "Розумне" розташування (вертикально)
$ #Cascade
# Каскадом
$ #LeftRight
# Зліва на право
$ #RightLeft
# Зправа на ліво
$ #TopBottom
# Зверху вниз
$ #BottomTop
# Знизу вверх
