$set 5 #Image

$ #ErrorCreatingSolidPixmap
# BImage::render_solid: ошибка создания pixmap\n
$ #ErrorCreatingXImage
# BImage::renderXImage: ошибка создания XImage\n
$ #UnsupVisual
# BImage::renderXImage: неподдерживаемый тип визуализации\n
$ #ErrorCreatingPixmap
# BImage::renderPixmap: ошибка создания pixmap\n
$ #InvalidColormapSize
# BImageControl::BImageControl: неверный размер таблицы цветов %d (%d/%d/%d) - уменьшается\n
$ #ErrorAllocatingColormap
# BImageControl::BImageControl: ошибка размещения таблицы цветов\n
$ #ColorAllocFail
# BImageControl::BImageControl: невозможно разместить цвет %d/%d/%d в памяти\n
$ #PixmapRelease
# BImageControl::~BImageControl: кэш точечных изображений - освобождается %d блоков\n
$ #PixmapCacheLarge
# BImageControl::renderImage: переполнение кэша, производится очистка\n
$ #ColorParseError
# BImageControl::getColor: ошибка разбора строки, описывающей цвет: '%s'\n
$ #ColorAllocError
# BImageControl::getColor: ошибка размещения цвета в памяти: '%s'\n
