$set 1 #BaseDisplay

$ #XError
# %s:  Napaka v sistemu oken X: %s(%d) operacijska koda %d/%d\n  vir 0x%lx\n
$ #SignalCaught
# %s: prejet signal %d\n
$ #ShuttingDown
# zaustavitev\n
$ #Aborting
# prekinitev... izmet posmrtnih ostankov core\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: neuspešna povezava do strežnika X.\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: povezave do prikazovalnika ni moč pripraviti do samodejne prekinitve ob zaustavitvi\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): pokvarjeno okno je odstranjeno iz dogodkovne vrste\n
