$set 15 #Common

$ #Yes
# Igen
$ #No
# Nem

$ #DirectionTitle
# Irány
$ #DirectionHoriz
# Vízszintes
$ #DirectionVert
# Függőleges

$ #AlwaysOnTop
# Mindig az előtérben

$ #PlacementTitle
# Elhelyezés
$ #PlacementTopLeft
# Bal felül
$ #PlacementCenterLeft
# Bal középen
$ #PlacementBottomLeft
# Bal lent
$ #PlacementTopCenter
# Középen felül
$ #PlacementBottomCenter
# Középen lent
$ #PlacementTopRight
# Jobb fent
$ #PlacementCenterRight
# Jobb középen
$ #PlacementBottomRight
# Jobb lent

$ #AutoHide
# Autómatikus elrejtés
