$set 15 #Common

$ #Yes
# 是
$ #No
# 否

$ #DirectionTitle
# 方向
$ #DirectionHoriz
# 水平
$ #DirectionVert
# 垂直

$ #AlwaysOnTop
# 总在最上面

$ #PlacementTitle
# 放置
$ #PlacementTopLeft
# 上左
$ #PlacementCenterLeft
# 中左
$ #PlacementBottomLeft
# 下左
$ #PlacementTopCenter
# 上中
$ #PlacementBottomCenter
# 下中
$ #PlacementTopRight
# 上右
$ #PlacementCenterRight
# 中右
$ #PlacementBottomRight
# 下右

$ #AutoHide
# 自动隐藏
