$set 3 #Configmenu

$ #ConfigOptions
# Instillinger
$ #FocusModel
# Fokus-modell 
$ #WindowPlacement
# Vindusplassering
$ #ImageDithering
# Bildeutjevning
$ #OpaqueMove
# Vis vindusinnhold ved flytting
$ #FullMax
# Full maksimering
$ #FocusNew
# Fokuser nye vinduer
$ #FocusLast
# Fokuser vindu ved skrivebords-bytte
$ #DisableBindings
# Slå av tastaturbindinger med Scroll Lock
$ #ClickToFocus
# Klikk for fokus
$ #SloppyFocus
# Slapp fokus
$ #AutoRaise
# Hev automatisk
$ #ClickRaise
# Klikk for å heve
$ #SmartRows
# Smart-plassering (rekker)
$ #SmartCols
# Smart-plassering (kolonner)
$ #Cascade
# Kaskade plassering
$ #LeftRight
# Fra venstre til høyre
$ #RightLeft
# Fra høyre til venstre
$ #TopBottom
# Topp til bunn
$ #BottomTop
# Bunn til topp
