$set 5 #Image

$ #ErrorCreatingSolidPixmap
# BImage::render_solid: kļūda veidojot pikseļkarti\n
$ #ErrorCreatingXImage
# BImage::renderXImage: kļūda veidojot XImage\n
$ #UnsupVisual
# BImage::renderXImage: neatbalstīts vizuāls\n
$ #ErrorCreatingPixmap
# BImage::renderPixmap: kļūda veidojot pikseļkarti\n
$ #InvalidColormapSize
# BImageControl::BImageControl: nederīgs krāsu kartes izmērs %d (%d/%d/%d) - reducēju\n
$ #ErrorAllocatingColormap
# BImageControl::BImageControl: kļūda piešķirot krāsu karti\n
$ #ColorAllocFail
# BImageControl::BImageControl: neizdevās piešķirt krāsu %d/%d/%d\n
$ #PixmapRelease
# BImageControl::~BImageControl: pikseļkartes kešs - atbrīvoju %d pikseļkartes\n
$ #PixmapCacheLarge
# BImageControl::renderImage: kešs ir liels, forsēju attīrīšanu\n
$ #ColorParseError
# BImageControl::getColor: krāsas parsēšanas kļūda: '%s'\n
$ #ColorAllocError
# BImageControl::getColor: krāsas piešķiršanas kļūda: '%s'\n
