$set 15 #Common

$ #Yes
# Да
$ #No
# Нет

$ #DirectionTitle
# Направление
$ #DirectionHoriz
# По горизонтали
$ #DirectionVert
# По вертикали

$ #AlwaysOnTop
# Всегда наверху

$ #PlacementTitle
# Местоположение
$ #PlacementTopLeft
# Слева вверху
$ #PlacementCenterLeft
# Слева по центру
$ #PlacementBottomLeft
# Слева внизу
$ #PlacementTopCenter
# Вверху по центру
$ #PlacementBottomCenter
# Внизу по центру
$ #PlacementTopRight
# Справа вверху
$ #PlacementCenterRight
# Справа по центру
$ #PlacementBottomRight
# Справа внизу

$ #AutoHide
# Скрывать автоматически
