$set 1 #BaseDisplay

$ #XError
# %s:  X erro: %s(%d) opcodes %d/%d\n  recurso 0x%lx\n
$ #SignalCaught
# %s: sinal recebido %d\n
$ #ShuttingDown
# finalizando\n
$ #Aborting
# abortando... descarregando memória\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: conecção ao servidor X falhou.\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: impossível exibir sinal de conexão como fechar-em-execução\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): removendo janela inválida da lista de eventos\n
