$set 3 #Configmenu

$ #ConfigOptions
# 配置选项
$ #FocusModel
# 聚焦模式
$ #WindowPlacement
# 窗口放置
$ #ImageDithering
# 图像调谐
$ #OpaqueMove
# 显示移动窗口内容
$ #FullMax
# 完全最大化
$ #FocusNew
# 聚焦新窗口
$ #FocusLast
# 桌面改变时聚焦窗口
$ #DisableBindings
# Scroll Lock取消热键
$ #ClickToFocus
# 点击聚焦
$ #SloppyFocus
# 自动聚焦
$ #AutoRaise
# 自动升起
$ #ClickRaise
# 点击升起
$ #SmartRows
# 自动放置(行)
$ #SmartCols
# 自动放置(列)
$ #Cascade
# 层叠放置
$ #LeftRight
# 从左到右
$ #RightLeft
# 从右到左
$ #TopBottom
# 从上到下
$ #BottomTop
# 从下到上
