$set 3 #Configmenu

$ #ConfigOptions
# 組態選項 
$ #FocusModel
# 指標模式設定
$ #WindowPlacement
# 視窗位置設定
$ #ImageDithering
# 圖像抖動瀘鏡
$ #OpaqueMove
# 視窗移動不透明設定 
$ #FullMax
# 視窗最大化至全螢幕 
$ #FocusNew
# 指標為新的視窗 
$ #FocusLast
# 當工作區變化自動指標至視窗
$ #ClickToFocus
# 點選老鼠於指標 
$ #SloppyFocus
# 自動指標當在視窗上面 
$ #AutoRaise
# 自動最上層 
$ #SmartRows
# 自動放置新的視窗位置 (行) 
$ #SmartCols
# 自動放置新的視窗位置 (列)
$ #Cascade
# 推疊的放置新的視窗位置 
$ #LeftRight
# 由左至右 
$ #RightLeft
# 由右至左 
$ #TopBottom
# 由上至下 
$ #BottomTop
# 由下至上 
