$set 3 #Configmenu

$ #ConfigOptions
# Konfiguračné voµby
$ #FocusModel
# Model zamerania
$ #WindowPlacement
# Umiestnenie okna
$ #ImageDithering
# Modifikácia farieb a jasu na obraze
$ #OpaqueMove
# Presúvanie celého okna
$ #FullMax
# Plná maximalizácia
$ #FocusNew
# Zamera» nové okná
$ #FocusLast
# Zamera» okno pri zmene pracovnej plochy
$ #DisableBindings
# Zakáza» väzby s klávesou Scroll Lock
$ #ClickToFocus
# Kliknú» pre zameranie
$ #SloppyFocus
# Nedbanlivé zameranie
$ #AutoRaise
# Automaticky presunú» do popredia
$ #ClickRaise
# Po kliknutí presunú» do popredia
$ #SmartRows
# Inteligentné umiestnenie (riadky)
$ #SmartCols
# Inteligentné umiestnenie (stĺpce)
$ #Cascade
# Kaskádové umiestnenie
$ #LeftRight
# Zµava doprava
$ #RightLeft
# Sprava doµava
$ #TopBottom
# Zhora nadol
$ #BottomTop
# Zdola nahor
