$set 15 #Common

$ #Yes
# Áno
$ #No
# Nie

$ #DirectionTitle
# Smer
$ #DirectionHoriz
# Horizontálny
$ #DirectionVert
# Vertikálny

$ #AlwaysOnTop
# Stále na vrchu

$ #PlacementTitle
# Umiestnenie
$ #PlacementTopLeft
# Vµavo hore
$ #PlacementCenterLeft
# Uprostred vµavo
$ #PlacementBottomLeft
# Vµavo dole
$ #PlacementTopCenter
# Hore uprostred
$ #PlacementBottomCenter
# Dole uprostred
$ #PlacementTopRight
# Vpravo hore
$ #PlacementCenterRight
# Uprostred vpravo
$ #PlacementBottomRight
# Vpravo dole

$ #AutoHide
# Automaticky skry»
