$set 3 #Configmenu

$ #ConfigOptions
# Конфигурация
$ #FocusModel
# Модель фокусировки
$ #WindowPlacement
# Расположение окон
$ #ImageDithering
# Сглаживать изображения
$ #OpaqueMove
# Показывать содержимое окон при перемещении
$ #FullMax
# Полная максимизация
$ #FocusNew
# Перемещать фокус на новые окна
$ #FocusLast
# Перемещать фокус при смене рабочего стола
$ #DisableBindings
# Scroll Lock отключает сочетания клавиш
$ #ClickToFocus
# Фокус по щелчку
$ #SloppyFocus
# Фокус по перемещению
$ #AutoRaise
# Автовсплытие
$ #ClickRaise
# Всплытие по щелчку
$ #SmartRows
# "Умное" расположение (по горизонтали)
$ #SmartCols
# "Умное" расположение (по вертикали)
$ #Cascade
# Каскадом
$ #LeftRight
# Слева направо
$ #RightLeft
# Справа налево
$ #TopBottom
# Сверху вниз
$ #BottomTop
# Снизу вверх
