$set 9 #Window


$ #Creating
# BlackboxWindow::BlackboxWindow: vytvára sa 0x%lx\n
$ #XGetWindowAttributesFail
# BlackboxWindow::BlackboxWindow: zlyhanie XGetWindowAttributres\n
$ #CannotFindScreen
# BlackboxWindow::BlackboxWindow: nie je možné nájsť obrazovku pre základné okno 0x%lx\n
$ #Unnamed
# Nepomenované
$ #MapRequest
# BlackboxWindow::mapRequestEvent() pre 0x%lx\n
$ #UnmapNotify
# BlackboxWindow::unmapNotifyEvent() pre 0x%lx\n
$ #ReparentNotify
# BlackboxWindow::reparentNotifyEvent: zmena potomka 0x%lx na rodiča 0x%lx\n
