$set 10 #Windowmenu

$ #SendTo
# Send til ...
$ #Shade
# Skygge
$ #Iconify
# Iconificeret
$ #Maximize
# Maksimer
$ #Raise
# Hæv
$ #Lower
# Sænk
$ #Stick
# Klæbrig
$ #KillClient
# Dræb klient
$ #Close
# Luk
