$set 5 #Image

$ #ErrorCreatingSolidPixmap
# BImage::render_solid: помилка створення мапи пікселів\n
$ #ErrorCreatingXImage
# BImage::renderXImage: помилка створення XImage\n
$ #UnsupVisual
# BImage::renderXImage: цей тип візуалізації не підтримується\n
$ #ErrorCreatingPixmap
# BImage::renderPixmap: помилка створення мапи пікселів\n
$ #InvalidColormapSize
# BImageControl::BImageControl: невірний розмір таблиці кольорів %d (%d/%d/%d) - зменшую\n
$ #ErrorAllocatingColormap
# BImageControl::BImageControl: помилка розміщення таблиці кольорів\n
$ #ColorAllocFail
# BImageControl::BImageControl: неможливо розмістити колір %d/%d/%d у пам'яті\n
$ #PixmapRelease
# BImageControl::~BImageControl: кеш пікселів зображень - звільняється %d блоків\n
$ #PixmapCacheLarge
# BImageControl::renderImage: кеш завеликий, примусове звільнення\n
$ #ColorParseError
# BImageControl::getColor: помилка розбору рядка, що описує колір: "%s"\n
$ #ColorAllocError
# BImageControl::getColor: помилка розміщення кольору у пам'яті: "%s"\n
