$set 16 #bsetroot

$ #MustSpecify
# %s: 错误: 必须从中指定一个: -solid, -mod, -gradient\n
$ #Usage
# %s 2.0: (c) 1997-2000 Brad Hughes\n\n\
	  (c) 2001-2002 Sean 'Shaleh' Perry\n\n\
  -display <string>        显示连接\n\
  -mod <x> <y>             模式\n\
  -foreground, -fg <color> 前景色\n\
  -background, -bg <color> 背景色\n\n\
  -gradient <texture>      渐进纹理\n\
  -from <color>            渐进开始色\n\
  -to <color>              渐进结束色\n\n\
  -solid <color>           单一色\n\n\
  -help                    显示这个帮助\n

