$set 15 #Common

$ #Yes
# Tak 
$ #No
# Nie

$ #DirectionTitle
# Kierunek
$ #DirectionHoriz
# Poziomy
$ #DirectionVert
# Pionowy

$ #AlwaysOnTop
# Zawsze na wierzchu

$ #PlacementTitle
# Położenie
$ #PlacementTopLeft
# Górne Lewe
$ #PlacementCenterLeft
# Środkowe Lewe
$ #PlacementBottomLeft
# Dolne Lewe
$ #PlacementTopCenter
# Górne Centralne
$ #PlacementBottomCenter
# Dolne Centralne
$ #PlacementTopRight
# Górne Prawe
$ #PlacementCenterRight
# Środkowe Prawe
$ #PlacementBottomRight
# Dolne Prawe

$ #AutoHide
# Automatyczne ukrywanie
