$set 10 #Windowmenu

$ #SendTo
# Küldés..
$ #Shade
# Shade
$ #Iconify
# Ikonizál
$ #Maximize
# Felnagyít
$ #Raise
# Előtérbe
$ #Lower
# Háttérbe
$ #Stick
# Mindenhol látható
$ #KillClient
# Kill
$ #Close
# Bezárás
