$set 10 #Windowmenu

$ #SendTo
# Відправити на ...
$ #Shade
# Згорнути у смужку
$ #Iconify
# Згорнути
$ #Maximize
# Розгорнути
$ #Raise
# Підняти наверх
$ #Lower
# Опустити вниз
$ #Stick
# Приклеїти
$ #KillClient
# Вбити програму
$ #Close
# Закрити
