$set 15 #Common

$ #Yes
# Oui
$ #No
# Non

$ #DirectionTitle
# Orientation
$ #DirectionHoriz
# Horizontal
$ #DirectionVert
# Vertical

$ #AlwaysOnTop
# Toujours au dessus

$ #PlacementTitle
# Disposition
$ #PlacementTopLeft
# En haut à gauche
$ #PlacementCenterLeft
# Au centre à gauche
$ #PlacementBottomLeft
# En bas à gauche
$ #PlacementTopCenter
# En haut au centre
$ #PlacementBottomCenter
# En bas au centre
$ #PlacementTopRight
# En haut à droite
$ #PlacementCenterRight
# Au centre à droite
$ #PlacementBottomRight
# En bas à droite

$ #AutoHide
# Auto disparaît
