$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: napaka pri poizvedovanju strežnika za X.\n  \
nek upravljalnik oken že teče v prikazovalniku %s.\n
$ #ManagingScreen
# BScreen::BScreen: upravljenje zaslona %d z videzem 0x%lx globine %d\n
$ #FontLoadFail
# BScreen::LoadStyle(): pisave '%s' ni moč naložiti\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): privzete pisave ni moč naložiti.\n
$ #EmptyMenuFile
# %s: prazna menujska datoteka\n
$ #xterm
# xterm
$ #Restart
# Ponovni zagon
$ #Exit
# Izhod
$ #EXECError
# BScreen::parseMenuFile: [exec] napaka: oznaki menuja ali ukaza nista določeni\n
$ #EXITError
# BScreen::parseMenuFile: [exit] napaka: oznaka menuja ni določena\n
$ #STYLEError
# BScreen::parseMenuFile: [style] napaka: oznaki menuja ali datoteke nista določeni\n
$ #CONFIGError
# BScreen::parseMenuFile: [config] napaka: oznaka menuja ni določena\n
$ #INCLUDEError
# BScreen::parseMenuFile: [include] napaka: ime datoteke ni določeno\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: [include] napaka: '%s' ni prava datoteka\n
$ #SUBMENUError
# BScreen::parseMenuFile: [submenu] napaka: oznaka menuja ni določena\n
$ #RESTARTError
# BScreen::parseMenuFile: [restart] napaka: oznaka menuja ni določena\n
$ #RECONFIGError
# BScreen::parseMenuFile: [reconfig] napaka: oznaka menuja ni določena\n
$ #STYLESDIRError
# BScreen::parseMenuFile: [stylesdir/stylesmenu] napaka: imenik ni določen\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: [stylesdir/stylesmenu] napaka: '%s' ni imenik\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: [stylesdir/stylesmenu] napaka:, '%s' ne obstaja\n
$ #WORKSPACESError
# BScreen::parseMenuFile: [workspaces] napaka: oznaka menuja ni določena\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# W: %4d x H: %4d

