$set 16 #bsetroot

$ #MustSpecify
# %s: ошибка: необходимо задать один из следующих ключей: -solid, -mod, -gradient\n
$ #Usage
# %s 2.0: (c) 1997-2000 Brad Hughes\n\n\
	  (c) 2001-2002 Sean 'Shaleh' Perry\n\n\
  -display <string>        использовать заданный дисплей\n\
  -mod <x> <y>             макет сетки\n\
  -foreground, -fg <color> цвет сетки\n\
  -background, -bg <color> цвет фона\n\n\
  -gradient <texture>      градиент\n\
  -from <color>            начальный цвет градиента\n\
  -to <color>              конечный цвет градиента\n\n\
  -solid <color>           сплошной цвет\n\n\
  -help                    вывести эту подсказку и выйти\n

