$set 14 #main

$ #RCRequiresArg
# fout: '-rc' heeft een argument nodig (bestandsnaam)\n
$ #DISPLAYRequiresArg
# fout: '-display' heeft een argument nodig (displaynaam)\n
$ #WarnDisplaySet
# waarschuwing: kan omgevingsvariabele 'DISPLAY' niet instellen\n
$ #Usage
# Blackbox %s: (c) 2001 - 2002 Sean 'Shaleh' Perry\n\
  \t\t\t 1997 - 2000, 2002 Brad Hughes\n\n\
  -display <displaynaam>   gebruik aangegeven display.\n\
  -rc <bestandsnaam>       gebruik alternatief resource bestand.\n\
  -version                 toon versie en stop.\n\
  -help                    toon deze hulp tekst en stop.\n\n
$ #CompileOptions
# Gecompileerd met instellingen:\n\
  Debugging:               %s\n\
  Shape:                   %s\n\
  8bpp Ordered Dithering:  %s\n\n
