$set 10 #Windowmenu

$ #SendTo
# Na delovno površino...
$ #Shade
# Zastri
$ #Iconify
# Pomanjšaj
$ #Maximize
# Povečaj
$ #Raise
# Prikaži
$ #Lower
# Zakrij
$ #Stick
# Prilepi
$ #KillClient
# Zaustavi 
$ #Close
# Zapusti
