$set 9 #Window


$ #Creating
# BlackboxWindow::BlackboxWindow: skapar 0x%lx\n
$ #XGetWindowAttributesFail
# BlackboxWindow::BlackboxWindow: XGetWindowAttributes misslyckades\n
$ #CannotFindScreen
# BlackboxWindow::BlackboxWindow: kan inte hitta skärm för rootfönster 0x%lx\n
$ #Unnamed
# Inget namn
$ #MapRequest
# BlackboxWindow::mapRequestEvent() för 0x%lx\n
$ #UnmapNotify
# BlackboxWindow::unmapNotifyEvent() för 0x%lx\n
$ #ReparentNotify
# BlackboxWindow::reparentNotifyEvent: återför 0x%lx till 0x%lx\n
