$set 10 #Windowmenu

$ #SendTo
# Sūtīt Uz ...
$ #Shade
# Ēnot
$ #Iconify
# Ikonificēt
$ #Maximize
# Maksimizēt
$ #Raise
# Pacelt
$ #Lower
# Pazemināt
$ #Stick
# Stiķēt
$ #KillClient
# Nokaut Klientu
$ #Close
# Aizvērt
