$set 9 #Window


$ #Creating
# BlackboxWindow::BlackboxWindow: створюється 0x%lx\n
$ #XGetWindowAttributesFail
# BlackboxWindow::BlackboxWindow: не вдалося отримати атрибути вікна (XGetWindowAttributres)\n
$ #CannotFindScreen
# BlackboxWindow::BlackboxWindow: нема можливості знайти екран для кореневого вікна 0x%lx\n
$ #Unnamed
# Неназване
$ #MapRequest
# BlackboxWindow::mapRequestEvent() для 0x%lx\n
$ #UnmapNotify
# BlackboxWindow::unmapNotifyEvent() для 0x%lx\n
$ #UnmapNotifyReparent
# BlackboxWindow::unmapnotifyEvent: reparent 0x%lx to root\n
